/*\t*******************************************************************
   Creation Date .......  Sat  05-06-1995  13:49:28
   Filename  ...........  timecb.cpp
   Project .............  mmsystem.h callback support test
   Author  .............  Matthew J. W. Ratcliff
   Language  ...........  C or C++
   Operating System  ...  Windows
   Prefix ..............  tiA
   Function:

 **   Copyright 1989-1995 by the McDonnell Douglas Corporation.       **
 **\t******************************************************************/

/*\r********************** Revision History ****************************
   Date    By           Change Description
dd-mmm-yy  nnn          text
---------  ----         -----------------------------------------------

**\r*/

/*\i******************** System Include Files *************************/
#ifndef NOSTRICT
#define STRICT // for MSVC users
#endif

#include <windows.h>
#include <mmsystem.h>
#include <ctl3d.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#ifndef max
#define max(a,b)    (((a) > (b)) ? (a) : (b))
#endif
#ifndef min
#define min(a,b)    (((a) < (b)) ? (a) : (b))
#endif

/********************** Constant Include Files ************************/
#include "timecb.h"
#include "timedll.h"
#include "liidSys.h"    /* System portability typedefs & constants    */
#include "liustrg.h"
/***************** External Variable Include Files ********************/

/***************** External Procedure Include Files *******************/
/*\i*/

/*\m********************** Module Constants ***************************/
// Initialization file
#define TIM_DEFAULT_FILE        "TIMECB.INI"
#define TIM_SLEN                240

#define TIM_TIMEDONE1            (WM_USER+1)
#define TIM_TIMEDONE10           (WM_USER+2)
#define TIM_TIMEDONEPERIODIC     (WM_USER+3)
#define TIM_RESOLUTION           1 // 1ms resolution please
/************************* Module Variables ***************************/
LOCAL const char *TIM_PROC_NAME="TIMECB";
LOCAL const char *TIM_ICON_NAME="TIMECBICO";

LOCAL char       *timMsg;
LOCAL BOOLEAN     timIsNew = FALSE_;
LOCAL HWND        timDialogHwnd = NULL;
LOCAL HWND        timDialogHlst  = NULL;
LOCAL UINT        timTimerId=NULL;
LOCAL UINT        timTimerRes;
/************************* Module Procedures **************************/

long FAR PASCAL WndProc
  ( HWND pasHwindow,
    WORD pasMessage,
    WORD pasWparam,
    LONG pasLparam );

long FAR PASCAL WndCommand
  ( HWND pasHwindow,
    WORD pasMessage,
    WORD pasWparam,
    LONG pasLparam );

/*\m*/
//////////////////////////////////////////////////////////
// NAME: setListboxMsg
// Add timer status messages to the list box.  Make sure
// the list box doesn't get too full.  This is an
// informational display only..user only cares about most
// recent data, so just scroll off old junk when the list
// starts getting pretty full.
//////////////////////////////////////////////////////////

void setListboxMsg( const char *pasMsg )
{
#define TIL_BIGLIST     400
LRESULT tilInx;
int     tilXnx;
SendMessage( timDialogHlst,LB_ADDSTRING,0,
            (LPARAM)pasMsg);
tilInx = SendMessage( timDialogHlst,LB_GETCOUNT, 0, 0);
if (tilInx > TIL_BIGLIST) // Lots of entries in the list?
  { // Clean some of it out, by rolling
    // some off the front of the list
    // Otherwise the list could get full and stop
    // adding our messages at the end
  for (tilXnx=0;tilXnx<TIL_BIGLIST/4;tilXnx++)
    {
    SendMessage( timDialogHlst, LB_DELETESTRING,
                 (WPARAM)0, 0);
    }
  tilInx = SendMessage( timDialogHlst,LB_GETCOUNT, 0, 0);
  }

SendMessage( timDialogHlst,LB_SETCURSEL, (WPARAM)tilInx-1, 0);
}

/*\p********************************************************************
**                                                                    **
    NAME:  WinMain

    PURPOSE:  to fire up the main application for the TIMECB
        application.

** By Matthew J. W. Ratcliff                                          **
**\p*******************************************************************/

int PASCAL WinMain( HINSTANCE pasInstance,
                    HINSTANCE pasPrevInstance,
                    LPSTR     pasCmdLine,
                    int       pasCmdShow )
{
/*********** Local Constant & Variable Declarations *******************/
int             tilOpenOk;
WNDCLASS        tilWndclass;
BOOLEAN         tilHwnd;
FARPROC         tilDlgProc;
STAT_TYPE       tilErr;
char           *tilBlank;
TIMECAPS        tilTc;
UINT            tilTres;
char            tilMsg[80];
/************************* Procedure Body *****************************/
tilHwnd     = NULL;
tilOpenOk   = FALSE_;
tilErr      = FAILED_;
if (!pasPrevInstance)
  {
  tilErr = SUCCEEDED_;
  if (timeGetDevCaps(&tilTc,sizeof(TIMECAPS)) != TIMERR_NOERROR)
    {
    // This occurs if the user hasn't installed any
    // multimedia drivers for a sound card or CD ROM,
    // for example.
    MessageBox( NULL, "Don't have system timer services",
                NULL, MB_OK | MB_ICONSTOP | MB_TASKMODAL );
    tilErr = FAILED_;
    }
  }
else
  {
  tilErr    = FAILED_; // Already running, bring to front
  tilOpenOk = FALSE_;
  MessageBeep(0);
  BringWindowToTop(FindWindow(TIM_PROC_NAME,NULL));
  }

if (tilErr == SUCCEEDED_)
  {
  tilTres = min(max(tilTc.wPeriodMin,TIM_RESOLUTION),tilTc.wPeriodMax);
  sprintf(tilMsg,"Timer Resolution Set to %dms per tic",
          (int)tilTres );
  timTimerRes = tilTres;
  MessageBox( NULL, tilMsg,"Ready", MB_OK | MB_ICONASTERISK );
  timeBeginPeriod(tilTres);
  tilWndclass.style           = CS_HREDRAW | CS_VREDRAW;
  tilWndclass.lpfnWndProc     = (WNDPROC)WndProc;
  tilWndclass.cbClsExtra      = 0;
  tilWndclass.cbWndExtra      = DLGWINDOWEXTRA;
  tilWndclass.hInstance       = pasInstance;
  tilWndclass.hIcon           = LoadIcon(pasInstance, TIM_ICON_NAME );
  tilWndclass.hCursor         = LoadCursor( NULL, IDC_ARROW );
  tilWndclass.hbrBackground   = (HBRUSH)GetStockObject(LTGRAY_BRUSH);
  tilWndclass.lpszMenuName    = NULL; // No menu
  tilWndclass.lpszClassName   = TIM_PROC_NAME;
  tilHwnd                     = RegisterClass( &tilWndclass );
  }

if (tilHwnd)
  {
  Ctl3dRegister( pasInstance );
  Ctl3dAutoSubclass( pasInstance );
  tilDlgProc = MakeProcInstance( (FARPROC)WndProc, pasInstance );
  DialogBox( pasInstance, TIM_PROC_NAME, NULL, (DLGPROC)tilDlgProc );
  tilOpenOk = TRUE_;
  Ctl3dUnregister( pasInstance );
  }
else
  {
  tilOpenOk = FALSE_;
  }
if (tilErr == SUCCEEDED_)
  {
  timeEndPeriod(tilTres);
  }
return(tilOpenOk);
} /* WinMain end */

/*\p********************************************************************
**                                                                    **
    NAME:  exitProgram

    PURPOSE:  to free storage and exit the program

** By Matthew J. W. Ratcliff                                          **
**\p*******************************************************************/

void exitProgram()
{
LIMDELA(timMsg);
PostQuitMessage(0);
}


/*\p********************************************************************
**                                                                    **
    NAME:  WndProc

    PURPOSE:  to handle window message processing for this
      dialog box application

** By Matthew J. W. Ratcliff                                          **
**   Return Val         type/expected/description                     **
**   ------------       --------------------------------------------  **
**   tilReturnVal       long, return value                            **
**\p*******************************************************************/

long FAR PASCAL WndProc
  ( HWND pasHwindow,
    WORD pasMessage,
    WORD pasWparam,
    LONG pasLparam )
{
/*********** Local Constant & Variable Declarations *******************/

static long           tilReturnVal      = NULL;
static HINSTANCE      tilHinstance      = NULL;
static STAT_TYPE      tilErr;
static DLGPROC        tilDlgProc;
char                  tilMsg[80];

/************************* Procedure Body *****************************/
tilReturnVal = 0;

switch ( pasMessage )
  {
  case WM_CREATE:
    tilHinstance = (HINSTANCE)((LPCREATESTRUCT)pasLparam)->hInstance;
    timMsg       = new char[TIM_SLEN];
    break;

  case WM_SYSCOLORCHANGE:
    Ctl3dColorChange();
    break;

  case WM_INITDIALOG:
    timDialogHwnd = pasHwindow;
    timDialogHlst = GetDlgItem( pasHwindow, IDC_LISTBOX );
    // setup dialog with initial values here, if needed
    Ctl3dSubclassDlg(pasHwindow, CTL3D_ALL);
    tilReturnVal = TRUE_;
    // Initialize everything
    // Can't stop until you've started
    EnableWindow(GetDlgItem(pasHwindow,IDC_PERIOD_STOP),FALSE);
    EnableWindow(GetDlgItem(pasHwindow,IDC_PERIOD_START),TRUE);
    break;

  case WM_CTLCOLOR:
    tilReturnVal = (long)(void far *)Ctl3dCtlColorEx( pasMessage,
                                                      pasWparam,
                                                      pasLparam );
    if (tilReturnVal == NULL)
      {
      tilReturnVal = DefWindowProc( pasHwindow,
                                    pasMessage,
                                    pasWparam,
                                    pasLparam );
      }
    break;
      
  case WM_COMMAND:
    tilReturnVal = WndCommand( pasHwindow,
                               pasMessage,
                               pasWparam,
                               pasLparam );
    break;

  case WM_DESTROY:
    exitProgram();
    break;

  default:
    tilReturnVal = DefWindowProc( pasHwindow,
                                  pasMessage,
                                  pasWparam,
                                  pasLparam );
    break;
  }

return(tilReturnVal);
} /* WndProc end */

/*\p********************************************************************
**                                                                    **
    NAME:  WndCommand

    PURPOSE:  to handle WM_COMMAND processing.

** By Matthew J. W. Ratcliff                                          **
**   Return Val         type/expected/description                     **
**   ------------       --------------------------------------------  **
**   tilReturnVal       long, return value                            **
**\p*******************************************************************/

long FAR PASCAL WndCommand
  ( HWND pasHwindow,
    WORD pasMessage,
    WORD pasWparam,
    LONG pasLparam )
{
/*********** Local Constant & Variable Declarations *******************/
long    tilReturnVal;
static long tilPcount;
static char tilMsg[80];
/************************* Procedure Body *****************************/
tilReturnVal = 0l;


switch( pasWparam )
  {
  case IDOK:
  case IDCANCEL: // exit asc tester
    // if periodic timer in use we MUST kill it before
    // termination
    if (timTimerId)
      {
      // Fake the stop periodic button press.
      PostMessage( pasHwindow, WM_COMMAND,
                   TIM_TIMEDONEPERIODIC, 0);
      }

    // Query to cancel changes before exiting, possibly
    EndDialog( pasHwindow, FALSE_);
    exitProgram();
    tilReturnVal = TRUE_;
    break;

  case IDC_WAIT_1SEC:
    setListboxMsg("1 second timer started...");
    // For a TIME_ONESHOT timer event we don't care about
    // the returned timer id, since after the oneshot timeout
    // occurs, the timer kills itself.
    timeSetEvent(1000,timTimerRes,OneMsTimeout,
                 (DWORD)(HWND far *)&timDialogHwnd,TIME_ONESHOT);
    break;
  case IDC_WAIT10SEC:
    setListboxMsg("10 second timer started...");
    timeSetEvent(10000,timTimerRes,TenMsTimeout,
                 (DWORD)(HWND far *)&timDialogHwnd,TIME_ONESHOT);
    break;
  case IDC_PERIOD_START:
    tilPcount = 0;
    // We must save the timerId so that we can safely kill it later.
    // You must kill a periodic timer before terminating, when
    // done using it or things will break badly.
    timTimerId = timeSetEvent(500,timTimerRes,FiveMsTimeout,
                              (DWORD)(HWND far *)&timDialogHwnd,TIME_PERIODIC);
    setListboxMsg("1/2 second periodic timer started...");
    // Can't start again until stop.
    // That is, we only support one 500ms periodic timer.
    // With a little more work you could easily support
    // multiple periodic timers.
    EnableWindow(GetDlgItem(pasHwindow,IDC_PERIOD_STOP),TRUE);
    EnableWindow(GetDlgItem(pasHwindow,IDC_PERIOD_START),FALSE);
    break;
  case IDC_PERIOD_STOP:
    timeKillEvent(timTimerId);
    timTimerId = NULL;
    // Periodic timer stopped. Can start again when ready.
    EnableWindow(GetDlgItem(pasHwindow,IDC_PERIOD_STOP),FALSE);
    EnableWindow(GetDlgItem(pasHwindow,IDC_PERIOD_START),TRUE);
    setListboxMsg("1/2 second periodic timer stopped.");
    break;

  case TIM_TIMEDONEPERIODIC:
    tilPcount++;
    sprintf(tilMsg,"Tick %ld, 1/2 second period done",
            tilPcount );
    setListboxMsg( tilMsg );
    break;

  case TIM_TIMEDONE1:
    setListboxMsg("1second timeout done");
    break;
  case TIM_TIMEDONE10:
    setListboxMsg("10second timeout done");
    break;
  default:
    tilReturnVal = DefWindowProc( pasHwindow,
                                  pasMessage,
                                  pasWparam,
                                  pasLparam );
    break;
  }

return(tilReturnVal);
} /* WndCommand end */

