/*\t*******************************************************************
   Creation Date .......  Sat  05-06-1995  13:49:28
   Filename  ...........  timedll.cpp
   Project .............  mmsystem.h callback support test
   Author  .............  Matthew J. W. Ratcliff
   Language  ...........  C or C++
   Operating System  ...  Windows
   Prefix ..............  tiA
   Function: Load fixed DLL segment for timer callback support.

 **   Copyright 1989-1995 by the McDonnell Douglas Corporation.       **
 **\t******************************************************************/

/*\r********************** Revision History ****************************
   Date    By           Change Description
dd-mmm-yy  nnn          text
---------  ----         -----------------------------------------------

**\r*/

/*\i******************** System Include Files *************************/

#include <windows.h>
#include <mmsystem.h>

/********************** Constant Include Files ************************/
#include "liidSys.h"    /* System portability typedefs & constants    */
#include "timedll.h"
/***************** External Variable Include Files ********************/

/***************** External Procedure Include Files *******************/
/*\i*/

/*\m********************** Module Constants ***************************/

/************************* Module Variables ***************************/
// Global Variable to store the DLL's Instance handle

static HANDLE ghDLLInst;

/************************* Module Procedures **************************/
// -----------------------------------------------------------------
//
// Function: LibMain
//
// Purpose : This is the DLL's entry point.  It is analogous to WinMain
//           for applications.
//
// Params  : hInstance   ==  The handle to the DLL's instance.
//           wDataSeg    ==  Basically it is a pointer to the DLL's
//                           data segment.
//           wHeapSize   ==  Size of the DLL's heap in bytes.
//           lpszCmdLine ==  The command line passed to the DLL
//                           by Windows.  This is rarely used.
//
// Returns : 1 indicating DLL initialization is successful.
//
// Comments: LibMain is called by Windows.  Do not call it in your
//           application!
// -----------------------------------------------------------------

int FAR PASCAL LibMain (HANDLE hInstance,
                        WORD   wDataSeg,
                        WORD   wHeapSize,
                        LPSTR  lpszCmdLine)
{
ghDLLInst = hInstance;

if (wHeapSize != 0)   // If DLL data seg is MOVEABLE
   UnlockData (0);

return (1);
}


// FROM DLLSKEL package on MSDN CD ROM
// -----------------------------------------------------------------
//
// WEP()
//
// Purpose:  Called by Windows just before Windows unloads the DLL.
//
// Parameters:  int  nParameter     -  This is a dummy parameter.
//                                     Windows expects this parameter
//                                     so be sure to include it.
//
// Return Value:  Always returns 1 to indicate success.
//
// Comments:  Every DLL MUST have a WEP.  WEP, as defined here, is the
//            proper way to define a WEP; do not do anything else
//            except return 1 because the WEP is called on the Windows
//            Kernel's stack, which is very small.
// -----------------------------------------------------------------

int FAR PASCAL WEP (int nParameter)
{
return (1);
}


//////////////////////////////////////////////////////////
// NAME: TenMsTimeout
// A 10 second timeout has occurred. The dwUser points
// to a window handle.  Post a WM_COMMAND message that
// a 10 second timeout, TIM_TIMEDONE10, has occurred.
//////////////////////////////////////////////////////////

void FAR PASCAL
TenMsTimeout(UINT wTimerID, UINT msg, DWORD dwUser, DWORD dw1, DWORD dw2)
{
HWND far *tilHptr;
tilHptr = (HWND far *)dwUser;
PostMessage( *tilHptr, WM_COMMAND, TIM_TIMEDONE10, 0);
}

//////////////////////////////////////////////////////////
// NAME: OneMsTimeout
// A 1 second timeout has occurred. The dwUser points
// to a window handle.  Post a WM_COMMAND message that
// a 1 second timeout, TIM_TIMEDONE1, has occurred.
//////////////////////////////////////////////////////////

void FAR PASCAL
OneMsTimeout(UINT wTimerID, UINT msg, DWORD dwUser, DWORD dw1, DWORD dw2)
{
HWND far *tilHptr;
tilHptr = (HWND far *)dwUser;
PostMessage( *tilHptr, WM_COMMAND, TIM_TIMEDONE1, 0);
}

//////////////////////////////////////////////////////////
// NAME: FiveMsTimeout
// A 1/2 second periodic timeout has occurred. The dwUser
// points to a window handle.  Post a WM_COMMAND message
// that a periodic timeout, TIM_TIMEDONEPERIODIC,
// has occurred.
//////////////////////////////////////////////////////////

void FAR PASCAL
FiveMsTimeout(UINT wTimerID, UINT msg, DWORD dwUser, DWORD dw1, DWORD dw2)
{
HWND far *tilHptr;
tilHptr = (HWND far *)dwUser;
PostMessage( *tilHptr, WM_COMMAND, TIM_TIMEDONEPERIODIC, 0);
}
